# Getting Started

## Installation

You can install the package using pip:

```shell
pip install pyobjectscript_gen
```

Or from source:

```shell
git clone https://github.com/Antoine-dh/pyobjectscript_gen.git
pip install .
```

## Basic Usage

Here's a basic example of how to use the library:

```python
from pyobjectscript_gen.cls import *

cls = Class(
    name="Demo.MyExample.MyClass",
    extends=["%RegisteredObject"]
)

# example of declaratively creating properties
cls.components = [
    Property(
        name="Property1",
        type="%String",
    ),
    Property(
        name="Property2",
        type="%Numeric",
    ),
]

# example of iteratively creating a method
method = Method("MyMethod")
method.return_type = "%String"
method.impl = [
    "set returnvalue=..Property1_..Property2",
    "quit returnvalue"
]

# append method to class components
cls.components.append(method)

# generate on sys.stdout
cls.generate()
```

See the generated class in [examples/generated/basic.cls](https://github.com/Antoine-dh/pyobjectscript_gen/blob/main/examples/generated/basic.cls)

## Running Examples

All examples code can be found in [examples/](https://github.com/Antoine-dh/pyobjectscript_gen/blob/main/examples/) and their respective output classes in [examples/generated/](https://github.com/Antoine-dh/pyobjectscript_gen/blob/main/examples/generated/)

```bash
# Basic class example with 2 properties and a method
python3 -m examples.basic > basic.cls

# Test class for every features capable to be generated by the library
python3 -m examples.features > features.cls

# Real life example of multiple classes generation using custom subclasses of the library's components
# Generates a simple REST example with a business operation, request and response classes in the specified directory
mkdir output/
python3 -m examples.rest ./output/
```

More examples to come...

---
# API Reference

## [Class][pyobjectscript_gen.cls.Class]
[Defining Classes](https://docs.intersystems.com/irislatest/csp/docbook/Doc.View.cls?KEY=GOBJ_classes)

---
# Objectscript Class Components

## [Component][pyobjectscript_gen.cls.Component]
Base class for Objectscript class components.

## [Parameter][pyobjectscript_gen.cls.Parameter]
[Defining Class Parameters](https://docs.intersystems.com/irislatest/csp/docbook/Doc.View.cls?KEY=GOBJ_parameters#GOBJ_parameters_def)

## [Property][pyobjectscript_gen.cls.Property]
[Defining and Using Literal Properties](https://docs.intersystems.com/irislatest/csp/docbook/DocBook.UI.Page.cls?KEY=GOBJ_proplit)

## [Method][pyobjectscript_gen.cls.Method]
[Defining Methods](https://docs.intersystems.com/irislatest/csp/docbook/DocBook.UI.Page.cls?KEY=GOBJ_methods)

## [ClassMethod][pyobjectscript_gen.cls.ClassMethod]
[Defining Methods](https://docs.intersystems.com/irislatest/csp/docbook/DocBook.UI.Page.cls?KEY=GOBJ_methods)

## [Projection][pyobjectscript_gen.cls.Projection]
[Defining Class Projections](https://docs.intersystems.com/irislatest/csp/docbook/DocBook.UI.Page.cls?KEY=GOBJ_projections)

## [Query][pyobjectscript_gen.cls.Query]
[Defining and Using Class Queries](https://docs.intersystems.com/irislatest/csp/docbook/DocBook.UI.Page.cls?KEY=GOBJ_queries)

## [Trigger][pyobjectscript_gen.cls.Trigger]
[Syntax of Triggers in Class Definitions](https://docs.intersystems.com/irislatest/csp/docbook/Doc.View.cls?KEY=ROBJ_trigger_syntax)

## [Index][pyobjectscript_gen.cls.Index]
[Defining Indexes Using a Class Definition](https://docs.intersystems.com/irislatest/csp/docbook/DocBook.UI.Page.cls?KEY=GSOD_indexes#GSOD_indexes_def_cls)

## [ForeignKey][pyobjectscript_gen.cls.ForeignKey]
[Syntax of Foreign Keys in Class Definitions](https://docs.intersystems.com/irislatest/csp/docbook/DocBook.UI.Page.cls?KEY=ROBJ_foreignkey_syntax)

## [Storage][pyobjectscript_gen.cls.Storage]
[Storage Definitions](https://docs.intersystems.com/irislatest/csp/docbook/Doc.View.cls?KEY=GOBJ_storage)

## [XData][pyobjectscript_gen.cls.XData]
[Defining and Using XData Blocks](https://docs.intersystems.com/irislatest/csp/docbook/Doc.View.cls?KEY=GOBJ_xdata)

---
# Miscellaneous classes

## [Expression][pyobjectscript_gen.cls.Expression]
Wrapper class for Objectscript expressions.

## [MethodArgument][pyobjectscript_gen.cls.MethodArgument]
Used to specify method arguments in the [Method][pyobjectscript_gen.cls.Method] or [Query][pyobjectscript_gen.cls.Query] component.
